

abstract class CollisionArea
{
	public abstract int left ();
	public abstract int right ();
	public abstract int top ();
	public abstract int bottom ();
	
	public boolean mightBeTouching (CollisionArea other)
	{
		return ((left() <= other.right())
			&& (right() >= other.left())
			&& (top() <= other.bottom())
			&& (bottom() >= other.top()));
	}
	
	public abstract void setPosition (DPoint p);
	
	public abstract boolean touching (CircularCollisionArea other);
	
	public abstract boolean touching (RectangularCollisionArea other);
	
	public static boolean touching (CollisionArea a, CollisionArea b)
	{
		if (b instanceof CircularCollisionArea)
		{
			return a.touching((CircularCollisionArea) b);
		}
		else if (b instanceof RectangularCollisionArea)
		{
			return a.touching((RectangularCollisionArea) b);
		}
		else
		{
			return a.mightBeTouching(b);
		}
	}
}

class CircularCollisionArea extends CollisionArea
{
	private DPoint position;
	private int radius;
	
	private int left;
	private int right;
	private int top;
	private int bottom;
	
	CircularCollisionArea (DPoint p, int radius)
	{
		this.radius = radius;
		
		setPosition(p);
	}
	
	public void setPosition (DPoint p)
	{
		position = new DPoint(p);
		update();
	}
		
	private void update ()
	{	
		left = position.x() - radius;
		right = position.x() + radius;
		top = position.y() - radius;
		bottom = position.y() + radius;
	}
	
	public DPoint getPosition ()
	{
		return new DPoint(position);
	}
	
	public int getRadius ()
	{
		return radius;
	}
	
	public void setRadius (int radius)
	{
		this.radius = radius;
		update();
	}
	
	public int left ()
	{
		return left;
	}
	
	public int right ()
	{
		return right;
	}
	
	public int top ()
	{
		return top;
	}
	
	public int bottom ()
	{
		return bottom;
	}
	
	public boolean touching (CircularCollisionArea other)
	{
		if (! mightBeTouching(other))
		{
			return false;
		}
		
		double distSquared = position.distanceSq(other.getPosition());
		
		int bothRadii = radius + other.getRadius();
		
		return (distSquared < bothRadii * bothRadii);
	}
	
	public boolean touching (RectangularCollisionArea other)
	{
		if (! mightBeTouching(other))
		{
			return false;
		}
		
		DPoint closest = other.getClosestPoint(position);
		
		double distSquared = position.distanceSq(closest);
		
		return (distSquared < radius * radius);
	}
}

class RectangularCollisionArea extends CollisionArea
{
	private int xOffset;
	private int yOffset;
	private int width;
	private int height;
	
	private int left;
	private int right;
	private int top;
	private int bottom;
	
	RectangularCollisionArea (DPoint p, int xOffset, int yOffset, int width, int height)
	{
		this.xOffset = xOffset;
		this.yOffset = yOffset;
		this.width = width;
		this.height = height;
		
		setPosition(p);
	}
	
	public void setPosition (DPoint p)
	{
		left = p.x() + xOffset;
		top = p.y() + yOffset;
		
		right = left + width;
		bottom = top + height;
	}
	
	public int left ()
	{
		return left;
	}
	
	public int right ()
	{
		return right;
	}
	
	public int top ()
	{
		return top;
	}
	
	public int bottom ()
	{
		return bottom;
	}
	
	public DPoint getClosestPoint (DPoint there)
	{
		int x = there.x();
		int y = there.y();
		
		if (x < left)
		{
			x = left;
		}
		else if (x > right)
		{
			x = right;
		}
		
		if (y < top)
		{
			y = top;
		}
		else if (y > bottom)
		{
			y = bottom;
		}
		
		return new DPoint(x, y);
	}
	
	public boolean touching (CircularCollisionArea other)
	{
		return other.touching(this);
	}
	
	public boolean touching (RectangularCollisionArea other)
	{
		return mightBeTouching(other);
	}
}
