import java.awt.*;
import java.util.*;

abstract class Human extends MovingEntity implements Damageable
{
	private double health;
	private double maxHealth;
	
	public static final double RUN = 0.2;
	public static final double WALK = 0.1;
	public static final double WANDER = 0.05;
	public static final double STILL = 0;
	public static final double BACK = -0.1;
	
	public final String type;
	
	private LinkedList <String[]> carrying = new LinkedList <String[]> ();
	
	Human (DPoint start, String type)
	{
		this(start, type, 25);
	}
	
	Human (DPoint start, String type, int health)
	{
		super(start, Layer.PEOPLE, 11);
		
		this.type = type;
		this.maxHealth = health;
		this.health = health;
		
		setSprite(Store.newSprite(type, false));
	}
	
	public boolean moveTowards (DPoint there, double moveSpeed, double turnSpeed, int time)
	{
		double diff = turnTowards(there, turnSpeed * time);
		
		if (diff > -5 && diff < 5)
		{
			return move(moveSpeed, time);
		}
		
		return true;
	}
	
	public boolean move (double distance)
	{
		throw(new RuntimeException());
	}
	
	public boolean move (double speed, int time)
	{
		double distance = speed * time;
		
		if (speed == 0)
		{
			sprite.stop();
		}
		else
		{
			sprite.start();
			
			sprite.setFrameTime((int) (12 / Math.abs(speed)));
		}
		
		double radians = Direction.toRadians(direction.getValue());
		
		double dx = Math.cos(radians) * distance;
		double dy = Math.sin(radians) * distance;
		
		boolean moveX =    valid(x + dx, y     );
		boolean moveY =    valid(x,      y + dy);
		boolean moveBoth = valid(x + dx, y + dy);
		
		boolean ret = true;
		
		if (! moveBoth)
		{
			if (Math.abs(dx) > Math.abs(dy))
			{
				// Moving more in x direction than y
				
				if (! moveX)
				{
					ret = false;
				}
				
				if (moveX && moveY)
				{
					moveY = false;
				}
			}
			else
			{
				// Moving more in y direction than x
				
				if (! moveY)
				{
					ret = false;
				}
				
				if (moveX && moveY)
				{
					moveX = false;
				}
			}
			
			if (! moveX)
			{
				dx = 0;
			}
			
			if (! moveY)
			{
				dy = 0;
			}
		}
		
		x += dx;
		y += dy;
		
		collision.setPosition(this);
		
		return ret;
	}
	
	public double getHealth ()
	{
		return health;
	}
	
	public double getMaxHealth ()
	{
		return maxHealth;
	}
	
	public void damage (double damage)
	{
		health -= damage;
		
		if (health <= 0)
		{
			health = 0;
			die();
		}
	}
	
	public void heal (double h)
	{
		health += h;
		
		if (health > maxHealth)
		{
			health = maxHealth;
		}
	}
	
	public boolean isDamaged ()
	{
		return (health < maxHealth);
	}
	
	public void die ()
	{
		DeadBody body = new DeadBody(this);
		
		GameArea.getLevel().addEntity(body);
		
		double d = direction.getValue() - Math.random() * 6;
		
		double offset = Direction.FULL / carrying.size();
		
		int dist = 6 + carrying.size() * 2;
		
		for (String[] o: carrying)
		{
			DPoint p = new DPoint(this);
			
			p.translate(dist, 0, d);
			
			d += offset;
			
			Entity e = Store.newEntity(o, p);
			
			GameArea.getLevel().addEntity(e);
		}
		
		GameArea.getLevel().removeEntity(this);
	}
	
	public void setCarrying (String[] object)
	{
		carrying.add(object);
	}
	
	public boolean canSee (DPoint there)
	{
		double dir = direction(there);
		
		double diff = direction.relative(dir);
		
		if ((diff < -250) || (diff > 250))
		{
			return false;
		}
		
		DPoint point = new DPoint(this);
		
		while (point.distance(there) > 16)
		{
			Tile tile = GameArea.getLevel().getTileAt(point);
			
			if (tile.type.equals("wall"))
			{
				return false;
			}
			
			point.translate(5, 0, dir);
		}
		
		return true;
	}
	
	public void cycle (int time)
	{
		wander(time);
	}
	
	private int turning = 0;
	private int time = 0;
	
	public void wander (int elapsed)
	{
		time -= elapsed;
		
		if (turning != 0)
		{
			if (time > 0)
			{
				direction.turnLeft(turning * elapsed);
			}
			else
			{
				time = 500 + (int) (Math.random() * 700);
				
				turning = 0;
			}
		}
		else
		{
			if ((time < 0) || ! move(WANDER, elapsed))
			{
				move(STILL, elapsed);
				
				time = 75 + (int) (Math.random() * 400);
				
				turning = (Math.random() < 0.5) ? -1 : 1;
			}
		}
	}
	
	public boolean solid ()
	{
		return true;
	}
}
