import java.awt.*;
import java.awt.geom.Point2D;
import java.util.*;

public class Level
{
	private Image image;
	private Tile[][] tiles;
	private Dimension tileSize;
	
	public final LinkedList <Entity> entities;
	
	private Queue <Entity> toAdd;
	private Queue <Entity> toRemove;
	
	public Level (Tile[][] tiles, String[] objects)
	{
		this.tiles = tiles;
		
		int rows = tiles.length;
		int cols = tiles[0].length;
		
		Image firstImage = tiles[0][0].image;
		
		int width = firstImage.getWidth(null);
		int height = firstImage.getHeight(null);
		
		tileSize = new Dimension(width, height);
		
		// Do we need a BufferedImage here?
		
		image = Store.newBufferedImage(width * cols, height * rows, false);
		
		Graphics g = image.getGraphics();
		
		for (int y = 0; y < tiles.length; y++)
		{
			for (int x = 0; x < tiles[y].length; x++)
			{
				g.drawImage(tiles[y][x].image, x * width, y * height, null);
			}
		}
		
		entities = new LinkedList <Entity> ();
		
		for (int i = 0; i < objects.length; i++)
		{
			String[] params = objects[i].split(",", 3);
			
			int x = Integer.parseInt(params[0]);
			int y = Integer.parseInt(params[1]);
			
			DPoint position = new DPoint(x, y);
			
			Entity entity = Store.newEntity(params[2].split(","), position);
			
			if (entity != null)
			{
				entities.add(entity);
			}
		}
		
		toAdd = new LinkedList <Entity> ();
		toRemove = new LinkedList <Entity> ();
	}
	
	public Tile[] getTiles (CollisionArea area)
	{
		int x1 = area.left() / tileSize.width;
		int x2 = area.right() / tileSize.width;
		int y1 = area.top() / tileSize.height;
		int y2 = area.bottom() / tileSize.height;
		
		int count = (x2 - x1 + 1) * (y2 - y1 + 1);
		
		Tile[] tiles = new Tile[count];
		
		int i = 0;
		
		for (int y = y1; y <= y2; y++)
		{
			for (int x = x1; x <= x2; x++)
			{
				tiles[i] = getTile(x, y);
				
				i++;
			}
		}
		
		return tiles;
	}
	
	public Tile getTile (int x, int y)
	{
		try {
		return tiles[y][x];
		} catch (Exception e) {return tiles[0][0];}
	}
	
	public Tile getTileAt (DPoint p)
	{
		return getTile(p.x() / tileSize.width, p.y() / tileSize.height);
	}
	
	public Image getImage ()
	{
		return image;
	}
	
	public void cycle (int time)
	{
		for (Entity entity: entities)
		{
			entity.cycle(time);
		}
	}
	
	public void draw (Graphics g, int x, int y)
	{
		g.drawImage(image, -x, -y, null);
		
		updateEntityList();
		
		for (Layer layer: Layer.values())
		{
			for (Entity entity: entities)
			{
				entity.draw(g, layer, x, y);
			}
		}
	}
	
	public void addEntity (Entity entity)
	{
		if (entity != null)
		{
			toAdd.offer(entity);
		}
	}
	
	public void removeEntity (Entity entity)
	{
		if (entity != null)
		{
			toRemove.offer(entity);
		}
	}
	
	public void updateEntityList ()
	{
		Entity entity;
		
		while ((entity = toAdd.poll()) != null)
		{
			entities.add(entity);
		}
		
		while ((entity = toRemove.poll()) != null)
		{
			entities.remove(entity);
		}
	}
}
